@@ -1,6 +1,5 @@
 """Module that implements ZMQ server communication over JSON-RPC 2.0 (https://www.jsonrpc.org/specification)."""
 from abc import ABC, abstractmethod
-import sys
 import os
 import json
 import logging
@@ -37,8 +36,6 @@ class BaseServer(ABC):
             start_services: Start services on initialization.
         """
         self.context = zmq_context
-        if sys.platform == "win32":
-            asyncio.set_event_loop_policy(asyncio.WindowsSelectorEventLoopPolicy())
         self.socket_ipc = self.context.socket(zmq.ROUTER)
         self.socket_ipc.setsockopt(zmq.LINGER, 0)
         ipc_uri = build_ipc_uri("self")
@@ -60,7 +57,7 @@ class BaseServer(ABC):
             address = await socket.recv()
             _ = await socket.recv()
             message = await socket.recv_string()
-            logger.info(f"Received request to {address}: {message}")
+            logger.trace(f"Received request to {address}: {message}")
             asyncio.create_task(self.handle_reply(socket, address, message))
 
     async def interrupt_loop(self):
@@ -183,14 +180,14 @@ class HTTPServer(BaseServer):
         self.app.on_startup.append(self.add_msg_loop_ipc)
         self.app.on_startup.append(self.add_interrupt_loop)
         logger.info("Starting server")
-        web.run_app(self.app, host="localhost", port=int(self.metadata.port))
+        web.run_app(self.app, host=self.metadata.host, port=int(self.metadata.port))
 
     async def handle_request(self, request):
         """Handle request."""
         try:
             address = request.match_info.get("name", "xxxxxxxxxxxx")
             message = await request.text()
-            logger.info(f"Received request to {address}: {message}")
+            logger.trace(f"Received request to {address}: {message}")
             response = await self.service_manager.handle_request(address, message)
         except Exception as e:
             response = {
